@@ -26,7 +26,6 @@ import (
 	"github.com/evanw/esbuild/internal/js_ast"
 	"github.com/evanw/esbuild/internal/js_lexer"
 	"github.com/evanw/esbuild/internal/js_parser"
-	"github.com/evanw/esbuild/internal/js_printer"
 	"github.com/evanw/esbuild/internal/logger"
 	"github.com/evanw/esbuild/internal/resolver"
 	"github.com/evanw/esbuild/internal/runtime"
@@ -314,7 +313,18 @@ func parseFile(args parseArgs) {
 		result.ok = true
 
 		// Mark that this file is from the "file" loader
-		result.file.inputFile.UniqueKeyForFileLoader = uniqueKey
+		result.file.inputFile.UniqueKeyForAdditionalFile = uniqueKey
+
+	case config.LoaderCopy:
+		uniqueKey := fmt.Sprintf("%sA%08d", args.uniqueKeyPrefix, args.sourceIndex)
+		uniqueKeyPath := uniqueKey + source.KeyPath.IgnoredSuffix
+		result.file.inputFile.Repr = &graph.CopyRepr{
+			URLForCode: uniqueKeyPath,
+		}
+		result.ok = true
+
+		// Mark that this file is from the "copy" loader
+		result.file.inputFile.UniqueKeyForAdditionalFile = uniqueKey
 
 	default:
 		var message string
@@ -358,9 +368,8 @@ func parseFile(args parseArgs) {
 
 	// Run the resolver on the parse thread so it's not run on the main thread.
 	// That way the main thread isn't blocked if the resolver takes a while.
-	if args.options.Mode == config.ModeBundle && !args.skipResolve {
+	if recordsPtr := result.file.inputFile.Repr.ImportRecords(); args.options.Mode == config.ModeBundle && !args.skipResolve && recordsPtr != nil {
 		// Clone the import records because they will be mutated later
-		recordsPtr := result.file.inputFile.Repr.ImportRecords()
 		records := append([]ast.ImportRecord{}, *recordsPtr...)
 		*recordsPtr = records
 		result.resolveResults = make([]*resolver.ResolveResult, len(records))
@@ -432,7 +441,7 @@ func parseFile(args parseArgs) {
 						text, suggestion, notes := ResolveFailureErrorTextSuggestionNotes(args.res, record.Path.Text, record.Kind,
 							pluginName, args.fs, absResolveDir, args.options.Platform, source.PrettyPath)
 						debug.LogErrorMsg(args.log, &source, record.Range, text, suggestion, notes)
-					} else if args.log.Level <= logger.LevelDebug && !didLogError && record.Flags.Has(ast.HandlesImportErrors) {
+					} else if !didLogError && record.Flags.Has(ast.HandlesImportErrors) {
 						args.log.AddIDWithNotes(logger.MsgID_Bundler_IgnoredDynamicImport, logger.Debug, &tracker, record.Range,
 							fmt.Sprintf("Importing %q was allowed even though it could not be resolved because dynamic import failures appear to be handled here:",
 								record.Path.Text), []logger.MsgData{tracker.MsgData(js_lexer.RangeOfIdentifier(source, record.ErrorHandlerLoc),
@@ -495,13 +504,17 @@ func ResolveFailureErrorTextSuggestionNotes(
 				hint = fmt.Sprintf("Use the relative path %q to reference the file %q. "+
 					"Without the leading \"./\", the path %q is being interpreted as a package path instead.",
 					"./"+path, res.PrettyPath(query.PathPair.Primary), path)
-				suggestion = string(js_printer.QuoteForJSON("./"+path, false))
+				suggestion = string(helpers.QuoteForJSON("./"+path, false))
 			}
 		}
 	}
 
 	if platform != config.PlatformNode {
-		if _, ok := resolver.BuiltInNodeModules[path]; ok {
+		pkg := path
+		if strings.HasPrefix(pkg, "node:") {
+			pkg = pkg[5:]
+		}
+		if resolver.BuiltInNodeModules[pkg] {
 			var how string
 			switch logger.API {
 			case logger.CLIAPI:
@@ -610,11 +623,12 @@ func extractSourceMapFromComment(
 			log.AddID(logger.MsgID_None, logger.Debug, &tracker, comment.Range, fmt.Sprintf("Failed to read file %q: %s", res.PrettyPath(path), originalError.Error()))
 		}
 		if err != nil {
+			kind := logger.Warning
 			if err == syscall.ENOENT {
 				// Don't report a warning because this is likely unactionable
-				return logger.Path{}, nil
+				kind = logger.Debug
 			}
-			log.AddID(logger.MsgID_SourceMap_MissingSourceMap, logger.Warning, &tracker, comment.Range,
+			log.AddID(logger.MsgID_SourceMap_MissingSourceMap, kind, &tracker, comment.Range,
 				fmt.Sprintf("Cannot read file %q: %s", res.PrettyPath(path), err.Error()))
 			return logger.Path{}, nil
 		}
@@ -1126,7 +1140,7 @@ func ScanBundle(
 	s.preprocessInjectedFiles()
 	entryPointMeta := s.addEntryPoints(entryPoints)
 	s.scanAllDependencies()
-	files := s.processScannedFiles()
+	files := s.processScannedFiles(entryPointMeta)
 
 	return Bundle{
 		fs:              fs,
@@ -1179,10 +1193,16 @@ func (s *scanner) maybeParseFile(
 
 	// Allow certain properties to be overridden
 	if len(resolveResult.JSXFactory) > 0 {
-		optionsClone.JSX.Factory = config.JSXExpr{Parts: resolveResult.JSXFactory}
+		optionsClone.JSX.Factory = config.DefineExpr{Parts: resolveResult.JSXFactory}
 	}
 	if len(resolveResult.JSXFragment) > 0 {
-		optionsClone.JSX.Fragment = config.JSXExpr{Parts: resolveResult.JSXFragment}
+		optionsClone.JSX.Fragment = config.DefineExpr{Parts: resolveResult.JSXFragment}
+	}
+	if resolveResult.JSX != config.TSJSXNone {
+		optionsClone.JSX.SetOptionsFromTSJSX(resolveResult.JSX)
+	}
+	if resolveResult.JSXImportSource != "" {
+		optionsClone.JSX.ImportSource = resolveResult.JSXImportSource
 	}
 	if resolveResult.UseDefineForClassFieldsTS != config.Unspecified {
 		optionsClone.UseDefineForClassFields = resolveResult.UseDefineForClassFieldsTS
@@ -1191,6 +1211,7 @@ func (s *scanner) maybeParseFile(
 		optionsClone.UnusedImportFlagsTS = resolveResult.UnusedImportFlagsTS
 	}
 	optionsClone.TSTarget = resolveResult.TSTarget
+	optionsClone.TSAlwaysStrict = resolveResult.TSAlwaysStrict
 
 	// Set the module type preference using node's module type rules
 	if strings.HasSuffix(path.Text, ".mjs") {
@@ -1615,6 +1636,11 @@ func lowestCommonAncestorDirectory(fs fs.FS, entryPoints []graph.EntryPoint) str
 					lastSlash = a
 				}
 			} else if boundaryA != boundaryB || unicode.ToLower(runeA) != unicode.ToLower(runeB) {
+				// If we're at the top-level directory, then keep the slash
+				if lastSlash < len(absDir) && !strings.ContainsAny(absDir[:lastSlash], "\\/") {
+					lastSlash++
+				}
+
 				// If both paths are different at this point, stop and set the lowest so
 				// far to the common parent directory. Compare using a case-insensitive
 				// comparison to handle paths on Windows.
@@ -1643,8 +1669,8 @@ func (s *scanner) scanAllDependencies() {
 		}
 
 		// Don't try to resolve paths if we're not bundling
-		if s.options.Mode == config.ModeBundle {
-			records := *result.file.inputFile.Repr.ImportRecords()
+		if recordsPtr := result.file.inputFile.Repr.ImportRecords(); s.options.Mode == config.ModeBundle && recordsPtr != nil {
+			records := *recordsPtr
 			for importRecordIndex := range records {
 				record := &records[importRecordIndex]
 
@@ -1690,12 +1716,18 @@ func (s *scanner) scanAllDependencies() {
 	}
 }
 
-func (s *scanner) processScannedFiles() []scannerFile {
+func (s *scanner) processScannedFiles(entryPointMeta []graph.EntryPoint) []scannerFile {
 	s.timer.Begin("Process scanned files")
 	defer s.timer.End("Process scanned files")
 
+	// Build a set of entry point source indices for quick lookup
+	entryPointSourceIndices := make(map[uint32]bool, len(entryPointMeta))
+	for _, meta := range entryPointMeta {
+		entryPointSourceIndices[meta.SourceIndex] = true
+	}
+
 	// Now that all files have been scanned, process the final file import records
-	for i, result := range s.results {
+	for sourceIndex, result := range s.results {
 		if !result.ok {
 			continue
 		}
@@ -1705,13 +1737,13 @@ func (s *scanner) processScannedFiles() []scannerFile {
 
 		// Begin the metadata chunk
 		if s.options.NeedsMetafile {
-			sb.Write(js_printer.QuoteForJSON(result.file.inputFile.Source.PrettyPath, s.options.ASCIIOnly))
+			sb.Write(helpers.QuoteForJSON(result.file.inputFile.Source.PrettyPath, s.options.ASCIIOnly))
 			sb.WriteString(fmt.Sprintf(": {\n      \"bytes\": %d,\n      \"imports\": [", len(result.file.inputFile.Source.Contents)))
 		}
 
 		// Don't try to resolve paths if we're not bundling
-		if s.options.Mode == config.ModeBundle {
-			records := *result.file.inputFile.Repr.ImportRecords()
+		if recordsPtr := result.file.inputFile.Repr.ImportRecords(); s.options.Mode == config.ModeBundle && recordsPtr != nil {
+			records := *recordsPtr
 			tracker := logger.MakeLineColumnTracker(&result.file.inputFile.Source)
 
 			for importRecordIndex := range records {
@@ -1743,6 +1775,7 @@ func (s *scanner) processScannedFiles() []scannerFile {
 				}
 
 				// Generate metadata about each import
+				otherFile := &s.results[record.SourceIndex.GetIndex()].file
 				if s.options.NeedsMetafile {
 					if isFirstImport {
 						isFirstImport = false
@@ -1751,14 +1784,13 @@ func (s *scanner) processScannedFiles() []scannerFile {
 						sb.WriteString(",\n        ")
 					}
 					sb.WriteString(fmt.Sprintf("{\n          \"path\": %s,\n          \"kind\": %s\n        }",
-						js_printer.QuoteForJSON(s.results[record.SourceIndex.GetIndex()].file.inputFile.Source.PrettyPath, s.options.ASCIIOnly),
-						js_printer.QuoteForJSON(record.Kind.StringForMetafile(), s.options.ASCIIOnly)))
+						helpers.QuoteForJSON(otherFile.inputFile.Source.PrettyPath, s.options.ASCIIOnly),
+						helpers.QuoteForJSON(record.Kind.StringForMetafile(), s.options.ASCIIOnly)))
 				}
 
 				switch record.Kind {
 				case ast.ImportAt, ast.ImportAtConditional:
 					// Using a JavaScript file with CSS "@import" is not allowed
-					otherFile := &s.results[record.SourceIndex.GetIndex()].file
 					if _, ok := otherFile.inputFile.Repr.(*graph.JSRepr); ok {
 						s.log.AddError(&tracker, record.Range,
 							fmt.Sprintf("Cannot import %q into a CSS file", otherFile.inputFile.Source.PrettyPath))
@@ -1769,7 +1801,6 @@ func (s *scanner) processScannedFiles() []scannerFile {
 
 				case ast.ImportURL:
 					// Using a JavaScript or CSS file with CSS "url()" is not allowed
-					otherFile := &s.results[record.SourceIndex.GetIndex()].file
 					switch otherRepr := otherFile.inputFile.Repr.(type) {
 					case *graph.CSSRepr:
 						s.log.AddError(&tracker, record.Range,
@@ -1783,11 +1814,18 @@ func (s *scanner) processScannedFiles() []scannerFile {
 					}
 				}
 
+				// If the imported file uses the "copy" loader, then move it from
+				// "SourceIndex" to "CopySourceIndex" so we don't end up bundling it.
+				if _, ok := otherFile.inputFile.Repr.(*graph.CopyRepr); ok {
+					record.CopySourceIndex = record.SourceIndex
+					record.SourceIndex = ast.Index32{}
+					continue
+				}
+
 				// If an import from a JavaScript file targets a CSS file, generate a
 				// JavaScript stub to ensure that JavaScript files only ever import
 				// other JavaScript files.
 				if _, ok := result.file.inputFile.Repr.(*graph.JSRepr); ok {
-					otherFile := &s.results[record.SourceIndex.GetIndex()].file
 					if css, ok := otherFile.inputFile.Repr.(*graph.CSSRepr); ok {
 						if s.options.WriteToStdout {
 							s.log.AddError(&tracker, record.Range,
@@ -1879,14 +1917,23 @@ func (s *scanner) processScannedFiles() []scannerFile {
 
 		result.file.jsonMetadataChunk = sb.String()
 
-		// If this file is from the "file" loader, generate an additional file
-		if result.file.inputFile.UniqueKeyForFileLoader != "" {
+		// If this file is from the "file" or "copy" loaders, generate an additional file
+		if result.file.inputFile.UniqueKeyForAdditionalFile != "" {
 			bytes := []byte(result.file.inputFile.Source.Contents)
+			template := s.options.AssetPathTemplate
+
+			// Use the entry path template instead of the asset path template if this
+			// file is an entry point and uses the "copy" loader. With the "file" loader
+			// the JS stub is the entry point, but with the "copy" loader the file is
+			// the entry point itself.
+			if result.file.inputFile.Loader == config.LoaderCopy && entryPointSourceIndices[uint32(sourceIndex)] {
+				template = s.options.EntryPathTemplate
+			}
 
 			// Add a hash to the file name to prevent multiple files with the same name
 			// but different contents from colliding
 			var hash string
-			if config.HasPlaceholder(s.options.AssetPathTemplate, config.HashPlaceholder) {
+			if config.HasPlaceholder(template, config.HashPlaceholder) {
 				h := xxhash.New()
 				h.Write(bytes)
 				hash = hashForFileName(h.Sum(nil))
@@ -1902,9 +1949,9 @@ func (s *scanner) processScannedFiles() []scannerFile {
 				/* customFilePath */ "",
 			)
 
-			// Apply the asset path template
+			// Apply the path template
 			templateExt := strings.TrimPrefix(originalExt, ".")
-			relPath := config.TemplateToString(config.SubstituteTemplate(s.options.AssetPathTemplate, config.PathPlaceholders{
+			relPath := config.TemplateToString(config.SubstituteTemplate(template, config.PathPlaceholders{
 				Dir:  &dir,
 				Name: &base,
 				Hash: &hash,
@@ -1915,7 +1962,7 @@ func (s *scanner) processScannedFiles() []scannerFile {
 			var jsonMetadataChunk string
 			if s.options.NeedsMetafile {
 				inputs := fmt.Sprintf("{\n        %s: {\n          \"bytesInOutput\": %d\n        }\n      }",
-					js_printer.QuoteForJSON(result.file.inputFile.Source.PrettyPath, s.options.ASCIIOnly),
+					helpers.QuoteForJSON(result.file.inputFile.Source.PrettyPath, s.options.ASCIIOnly),
 					len(bytes),
 				)
 				jsonMetadataChunk = fmt.Sprintf(
@@ -1933,7 +1980,7 @@ func (s *scanner) processScannedFiles() []scannerFile {
 			}}
 		}
 
-		s.results[i] = result
+		s.results[sourceIndex] = result
 	}
 
 	// The linker operates on an array of files, so construct that now. This
@@ -2253,9 +2300,13 @@ func findReachableFiles(files []graph.InputFile, entryPoints []graph.EntryPoint)
 			if repr, ok := file.Repr.(*graph.JSRepr); ok && repr.CSSSourceIndex.IsValid() {
 				visit(repr.CSSSourceIndex.GetIndex())
 			}
-			for _, record := range *file.Repr.ImportRecords() {
-				if record.SourceIndex.IsValid() {
-					visit(record.SourceIndex.GetIndex())
+			if recordsPtr := file.Repr.ImportRecords(); recordsPtr != nil {
+				for _, record := range *recordsPtr {
+					if record.SourceIndex.IsValid() {
+						visit(record.SourceIndex.GetIndex())
+					} else if record.CopySourceIndex.IsValid() {
+						visit(record.CopySourceIndex.GetIndex())
+					}
 				}
 			}
 
@@ -2312,7 +2363,7 @@ func (b *Bundle) computeDataForSourceMapsInParallel(options *config.Options, rea
 				if !options.ExcludeSourcesContent {
 					if sm == nil {
 						// Simple case: no nested source map
-						result.quotedContents = [][]byte{js_printer.QuoteForJSON(f.inputFile.Source.Contents, options.ASCIIOnly)}
+						result.quotedContents = [][]byte{helpers.QuoteForJSON(f.inputFile.Source.Contents, options.ASCIIOnly)}
 					} else {
 						// Complex case: nested source map
 						result.quotedContents = make([][]byte, len(sm.Sources))
@@ -2324,7 +2375,7 @@ func (b *Bundle) computeDataForSourceMapsInParallel(options *config.Options, rea
 								if value := sm.SourcesContent[i]; value.Quoted != "" {
 									if options.ASCIIOnly && !isASCIIOnly(value.Quoted) {
 										// Re-quote non-ASCII values if output is ASCII-only
-										quotedContents = js_printer.QuoteForJSON(helpers.UTF16ToString(value.Value), options.ASCIIOnly)
+										quotedContents = helpers.QuoteForJSON(helpers.UTF16ToString(value.Value), options.ASCIIOnly)
 									} else {
 										// Otherwise just use the value directly from the input file
 										quotedContents = []byte(value.Quoted)
@@ -2386,7 +2437,7 @@ func (b *Bundle) generateMetadataJSON(results []graph.OutputFile, allReachableFi
 				sb.WriteString(",\n    ")
 			}
 			paths[path] = true
-			sb.WriteString(fmt.Sprintf("%s: ", js_printer.QuoteForJSON(path, asciiOnly)))
+			sb.WriteString(fmt.Sprintf("%s: ", helpers.QuoteForJSON(path, asciiOnly)))
 			sb.WriteString(result.JSONMetadataChunk)
 		}
 	}
