@@ -3,6 +3,7 @@ package bundler
 import (
 	"testing"
 
+	"github.com/evanw/esbuild/internal/compat"
 	"github.com/evanw/esbuild/internal/config"
 )
 
@@ -82,14 +83,14 @@ func TestCSSAtImportExternal(t *testing.T) {
 		options: config.Options{
 			Mode:         config.ModeBundle,
 			AbsOutputDir: "/out",
-			ExternalModules: config.ExternalModules{
-				AbsPaths: map[string]bool{
+			ExternalSettings: config.ExternalSettings{
+				PostResolve: config.ExternalMatchers{Exact: map[string]bool{
 					"/external1.css": true,
 					"/external2.css": true,
 					"/external3.css": true,
 					"/external4.css": true,
 					"/external5.css": true,
-				},
+				}},
 			},
 		},
 	})
@@ -160,7 +161,8 @@ func TestCSSFromJSMissingStarImport(t *testing.T) {
 			Mode:         config.ModeBundle,
 			AbsOutputDir: "/out",
 		},
-		expectedCompileLog: `entry.js: WARNING: Import "missing" will always be undefined because there is no matching export in "a.css"
+		debugLogs: true,
+		expectedCompileLog: `entry.js: DEBUG: Import "missing" will always be undefined because there is no matching export in "a.css"
 `,
 	})
 }
@@ -295,10 +297,10 @@ func TestExternalImportURLInCSS(t *testing.T) {
 		options: config.Options{
 			Mode:         config.ModeBundle,
 			AbsOutputDir: "/out",
-			ExternalModules: config.ExternalModules{
-				AbsPaths: map[string]bool{
+			ExternalSettings: config.ExternalSettings{
+				PostResolve: config.ExternalMatchers{Exact: map[string]bool{
 					"/src/external.png": true,
-				},
+				}},
 			},
 		},
 	})
@@ -651,10 +653,10 @@ func TestCSSExternalQueryAndHashNoMatchIssue1822(t *testing.T) {
 		options: config.Options{
 			Mode:          config.ModeBundle,
 			AbsOutputFile: "/out.css",
-			ExternalModules: config.ExternalModules{
-				Patterns: []config.WildcardPattern{
+			ExternalSettings: config.ExternalSettings{
+				PreResolve: config.ExternalMatchers{Patterns: []config.WildcardPattern{
 					{Suffix: ".png"},
-				},
+				}},
 			},
 		},
 		expectedScanLog: `entry.css: ERROR: Could not resolve "foo/bar.png?baz"
@@ -677,12 +679,31 @@ func TestCSSExternalQueryAndHashMatchIssue1822(t *testing.T) {
 		options: config.Options{
 			Mode:          config.ModeBundle,
 			AbsOutputFile: "/out.css",
-			ExternalModules: config.ExternalModules{
-				Patterns: []config.WildcardPattern{
+			ExternalSettings: config.ExternalSettings{
+				PreResolve: config.ExternalMatchers{Patterns: []config.WildcardPattern{
 					{Suffix: ".png?baz"},
 					{Suffix: ".png#baz"},
-				},
+				}},
 			},
 		},
 	})
 }
+
+func TestCSSNestingOldBrowser(t *testing.T) {
+	css_suite.expectBundled(t, bundled{
+		files: map[string]string{
+			"/entry.css": `
+				a { &:hover { color: red; } }
+			`,
+		},
+		entryPaths: []string{"/entry.css"},
+		options: config.Options{
+			Mode:                   config.ModeBundle,
+			AbsOutputFile:          "/out.css",
+			UnsupportedCSSFeatures: compat.Nesting,
+			OriginalTargetEnv:      "chrome10",
+		},
+		expectedScanLog: `entry.css: WARNING: CSS nesting syntax is not supported in the configured target environment (chrome10)
+`,
+	})
+}
