@@ -3014,6 +3014,58 @@ const-update.js: NOTE: The symbol "x" was declared a constant here:
 	})
 }
 
+func TestConstValueInliningDirectEval(t *testing.T) {
+	dce_suite.expectBundled(t, bundled{
+		files: map[string]string{
+			"/top-level-no-eval.js": `
+				const x = 1
+				console.log(x, evil('x'))
+			`,
+			"/top-level-eval.js": `
+				const x = 1
+				console.log(x, eval('x'))
+			`,
+			"/nested-no-eval.js": `
+				(() => {
+					const x = 1
+					console.log(x, evil('x'))
+				})()
+			`,
+			"/nested-eval.js": `
+				(() => {
+					const x = 1
+					console.log(x, eval('x'))
+				})()
+			`,
+			"/ts-namespace-no-eval.ts": `
+				namespace y {
+					export const x = 1
+					console.log(x, evil('x'))
+				}
+			`,
+			"/ts-namespace-eval.ts": `
+				namespace z {
+					export const x = 1
+					console.log(x, eval('x'))
+				}
+			`,
+		},
+		entryPaths: []string{
+			"/top-level-no-eval.js",
+			"/top-level-eval.js",
+			"/nested-no-eval.js",
+			"/nested-eval.js",
+			"/ts-namespace-no-eval.ts",
+			"/ts-namespace-eval.ts",
+		},
+		options: config.Options{
+			Mode:         config.ModePassThrough,
+			AbsOutputDir: "/out",
+			MinifySyntax: true,
+		},
+	})
+}
+
 func TestCrossModuleConstantFolding(t *testing.T) {
 	dce_suite.expectBundled(t, bundled{
 		files: map[string]string{
@@ -3219,3 +3271,175 @@ func TestMultipleDeclarationTreeShakingMinifySyntax(t *testing.T) {
 		},
 	})
 }
+
+// Pure call removal should still run iterators, which can have side effects
+func TestPureCallsWithSpread(t *testing.T) {
+	dce_suite.expectBundled(t, bundled{
+		files: map[string]string{
+			"/entry.js": `
+				/* @__PURE__ */ foo(...args);
+				/* @__PURE__ */ new foo(...args);
+			`,
+		},
+		entryPaths: []string{"/entry.js"},
+		options: config.Options{
+			Mode:          config.ModeBundle,
+			AbsOutputFile: "/out.js",
+			MinifySyntax:  true,
+		},
+	})
+}
+
+func TestTopLevelFunctionInliningWithSpread(t *testing.T) {
+	dce_suite.expectBundled(t, bundled{
+		files: map[string]string{
+			"/entry.js": `
+				function empty1() {}
+				function empty2() {}
+				function empty3() {}
+
+				function identity1(x) { return x }
+				function identity2(x) { return x }
+				function identity3(x) { return x }
+
+				empty1()
+				empty2(args)
+				empty3(...args)
+
+				identity1()
+				identity2(args)
+				identity3(...args)
+			`,
+
+			"/inner.js": `
+				export function empty1() {}
+				export function empty2() {}
+				export function empty3() {}
+
+				export function identity1(x) { return x }
+				export function identity2(x) { return x }
+				export function identity3(x) { return x }
+			`,
+
+			"/entry-outer.js": `
+				import {
+					empty1,
+					empty2,
+					empty3,
+
+					identity1,
+					identity2,
+					identity3,
+				} from './inner.js'
+
+				empty1()
+				empty2(args)
+				empty3(...args)
+
+				identity1()
+				identity2(args)
+				identity3(...args)
+			`,
+		},
+		entryPaths: []string{"/entry.js", "/entry-outer.js"},
+		options: config.Options{
+			Mode:         config.ModeBundle,
+			AbsOutputDir: "/out",
+			MinifySyntax: true,
+		},
+	})
+}
+
+func TestNestedFunctionInliningWithSpread(t *testing.T) {
+	dce_suite.expectBundled(t, bundled{
+		files: map[string]string{
+			"/entry.js": `
+				function empty1() {}
+				function empty2() {}
+				function empty3() {}
+
+				function identity1(x) { return x }
+				function identity2(x) { return x }
+				function identity3(x) { return x }
+
+				check(
+					empty1(),
+					empty2(args),
+					empty3(...args),
+
+					identity1(),
+					identity2(args),
+					identity3(...args),
+				)
+			`,
+
+			"/inner.js": `
+				export function empty1() {}
+				export function empty2() {}
+				export function empty3() {}
+
+				export function identity1(x) { return x }
+				export function identity2(x) { return x }
+				export function identity3(x) { return x }
+			`,
+
+			"/entry-outer.js": `
+				import {
+					empty1,
+					empty2,
+					empty3,
+
+					identity1,
+					identity2,
+					identity3,
+				} from './inner.js'
+
+				check(
+					empty1(),
+					empty2(args),
+					empty3(...args),
+
+					identity1(),
+					identity2(args),
+					identity3(...args),
+				)
+			`,
+		},
+		entryPaths: []string{"/entry.js", "/entry-outer.js"},
+		options: config.Options{
+			Mode:         config.ModeBundle,
+			AbsOutputDir: "/out",
+			MinifySyntax: true,
+		},
+	})
+}
+
+func TestPackageJsonSideEffectsFalseCrossPlatformSlash(t *testing.T) {
+	dce_suite.expectBundled(t, bundled{
+		files: map[string]string{
+			"/Users/user/project/src/entry.js": `
+				import "demo-pkg/foo"
+				import "demo-pkg/bar"
+			`,
+			"/Users/user/project/node_modules/demo-pkg/foo.js": `
+				console.log('foo')
+			`,
+			"/Users/user/project/node_modules/demo-pkg/bar/index.js": `
+				console.log('bar')
+			`,
+			"/Users/user/project/node_modules/demo-pkg/package.json": `
+				{
+					"sideEffects": [
+						"**/foo.js",
+						"bar/index.js"
+					]
+				}
+			`,
+		},
+		entryPaths: []string{"/Users/user/project/src/entry.js"},
+		options: config.Options{
+			Mode:          config.ModeBundle,
+			AbsOutputFile: "/out.js",
+		},
+	})
+}
