@@ -110,12 +110,21 @@ func RealFS(options RealFSOptions) (FS, error) {
 		watchData = make(map[string]privateWatchData)
 	}
 
-	return &realFS{
+	var result FS = &realFS{
 		entries:           make(map[string]entriesOrErr),
 		fp:                fp,
 		watchData:         watchData,
 		doNotCacheEntries: options.DoNotCache,
-	}, nil
+	}
+
+	// Add a wrapper that lets us traverse into ".zip" files. This is what yarn
+	// uses as a package format when in yarn is in its "PnP" mode.
+	result = &zipFS{
+		inner:    result,
+		zipFiles: make(map[string]*zipFile),
+	}
+
+	return result, nil
 }
 
 func (fs *realFS) ReadDirectory(dir string) (entries DirEntries, canonicalError error, originalError error) {
@@ -340,11 +349,12 @@ func (fs *realFS) readdir(dirname string) (entries []string, canonicalError erro
 	}
 
 	defer f.Close()
-	entries, err := f.Readdirnames(-1)
+	entries, originalError = f.Readdirnames(-1)
+	canonicalError = originalError
 
 	// Unwrap to get the underlying error
-	if syscallErr, ok := err.(*os.SyscallError); ok {
-		err = syscallErr.Unwrap()
+	if syscallErr, ok := canonicalError.(*os.SyscallError); ok {
+		canonicalError = syscallErr.Unwrap()
 	}
 
 	// Don't convert ENOTDIR to ENOENT here. ENOTDIR is a legitimate error
@@ -485,13 +495,13 @@ func (fs *realFS) WatchData() WatchData {
 					}
 				} else {
 					// Check individual entries
-					isPresent := make(map[string]bool, len(names))
+					lookup := make(map[string]string, len(names))
 					for _, name := range names {
-						isPresent[strings.ToLower(name)] = true
+						lookup[strings.ToLower(name)] = name
 					}
 					for name, wasPresent := range data.accessedEntries.wasPresent {
-						if wasPresent != isPresent[name] {
-							return fs.Join(path, name)
+						if originalName, isPresent := lookup[name]; wasPresent != isPresent {
+							return fs.Join(path, originalName)
 						}
 					}
 				}
