@@ -187,3 +187,34 @@ func ParseJSON(log logger.Log, source logger.Source, options JSONOptions) (resul
 	p.lexer.Expect(js_lexer.TEndOfFile)
 	return
 }
+
+func isValidJSON(value js_ast.Expr) bool {
+	switch e := value.Data.(type) {
+	case *js_ast.ENull, *js_ast.EBoolean, *js_ast.EString, *js_ast.ENumber:
+		return true
+
+	case *js_ast.EArray:
+		for _, item := range e.Items {
+			if !isValidJSON(item) {
+				return false
+			}
+		}
+		return true
+
+	case *js_ast.EObject:
+		for _, property := range e.Properties {
+			if property.Kind != js_ast.PropertyNormal || property.Flags&(js_ast.PropertyIsComputed|js_ast.PropertyIsMethod) != 0 {
+				return false
+			}
+			if _, ok := property.Key.Data.(*js_ast.EString); !ok {
+				return false
+			}
+			if !isValidJSON(property.ValueOrNil) {
+				return false
+			}
+		}
+		return true
+	}
+
+	return false
+}
