@@ -17,6 +17,7 @@ import (
 )
 
 type packageJSON struct {
+	name           string
 	mainFields     map[string]mainField
 	moduleTypeData js_ast.ModuleTypeData
 
@@ -273,6 +274,13 @@ func (r resolverQuery) parsePackageJSON(inputPath string) *packageJSON {
 		mainFields: make(map[string]mainField),
 	}
 
+	// Read the "name" field
+	if nameJSON, _, ok := getProperty(json, "name"); ok {
+		if nameValue, ok := getString(nameJSON); ok {
+			packageJSON.name = nameValue
+		}
+	}
+
 	// Read the "type" field
 	if typeJSON, typeKeyLoc, ok := getProperty(json, "type"); ok {
 		if typeValue, ok := getString(typeJSON); ok {
@@ -414,6 +422,7 @@ func (r resolverQuery) parsePackageJSON(inputPath string) *packageJSON {
 					pattern = "**/" + pattern
 				}
 				absPattern := r.fs.Join(inputPath, pattern)
+				absPattern = strings.ReplaceAll(absPattern, "\\", "/") // Avoid problems with Windows-style slashes
 				re, hadWildcard := globstarToEscapedRegexp(absPattern)
 
 				// Wildcard patterns require more expensive matching
