@@ -97,8 +97,10 @@ func validatePathTemplate(template string) []config.PathTemplate {
 	return parts
 }
 
-func validatePlatform(value Platform) config.Platform {
+func validatePlatform(value Platform, defaultPlatform config.Platform) config.Platform {
 	switch value {
+	case PlatformDefault:
+		return defaultPlatform
 	case PlatformBrowser:
 		return config.PlatformBrowser
 	case PlatformNode:
@@ -229,30 +231,32 @@ func validateTreeShaking(value TreeShaking, bundle bool, format Format) bool {
 
 func validateLoader(value Loader) config.Loader {
 	switch value {
-	case LoaderNone:
-		return config.LoaderNone
-	case LoaderJS:
-		return config.LoaderJS
-	case LoaderJSX:
-		return config.LoaderJSX
-	case LoaderTS:
-		return config.LoaderTS
-	case LoaderTSX:
-		return config.LoaderTSX
-	case LoaderJSON:
-		return config.LoaderJSON
-	case LoaderText:
-		return config.LoaderText
 	case LoaderBase64:
 		return config.LoaderBase64
+	case LoaderBinary:
+		return config.LoaderBinary
+	case LoaderCopy:
+		return config.LoaderCopy
+	case LoaderCSS:
+		return config.LoaderCSS
 	case LoaderDataURL:
 		return config.LoaderDataURL
 	case LoaderFile:
 		return config.LoaderFile
-	case LoaderBinary:
-		return config.LoaderBinary
-	case LoaderCSS:
-		return config.LoaderCSS
+	case LoaderJS:
+		return config.LoaderJS
+	case LoaderJSON:
+		return config.LoaderJSON
+	case LoaderJSX:
+		return config.LoaderJSX
+	case LoaderNone:
+		return config.LoaderNone
+	case LoaderText:
+		return config.LoaderText
+	case LoaderTS:
+		return config.LoaderTS
+	case LoaderTSX:
+		return config.LoaderTSX
 	case LoaderDefault:
 		return config.LoaderDefault
 	default:
@@ -327,26 +331,7 @@ func validateFeatures(log logger.Log, target Target, engines []Engine) (config.T
 				if patch, err := strconv.Atoi(match[3]); err == nil {
 					version = append(version, patch)
 				}
-				switch engine.Name {
-				case EngineChrome:
-					constraints[compat.Chrome] = version
-				case EngineEdge:
-					constraints[compat.Edge] = version
-				case EngineFirefox:
-					constraints[compat.Firefox] = version
-				case EngineIE:
-					constraints[compat.IE] = version
-				case EngineIOS:
-					constraints[compat.IOS] = version
-				case EngineNode:
-					constraints[compat.Node] = version
-				case EngineOpera:
-					constraints[compat.Opera] = version
-				case EngineSafari:
-					constraints[compat.Safari] = version
-				default:
-					panic("Invalid engine name")
-				}
+				constraints[convertEngineName(engine.Name)] = version
 				continue
 			}
 		}
@@ -373,6 +358,30 @@ func validateFeatures(log logger.Log, target Target, engines []Engine) (config.T
 	return targetFromAPI, compat.UnsupportedJSFeatures(constraints), compat.UnsupportedCSSFeatures(constraints), targetEnv
 }
 
+func validateSupported(log logger.Log, supported map[string]bool) (
+	jsFeature compat.JSFeature,
+	jsMask compat.JSFeature,
+	cssFeature compat.CSSFeature,
+	cssMask compat.CSSFeature,
+) {
+	for k, v := range supported {
+		if js, ok := compat.StringToJSFeature[k]; ok {
+			jsMask |= js
+			if !v {
+				jsFeature |= js
+			}
+		} else if css, ok := compat.StringToCSSFeature[k]; ok {
+			cssMask |= css
+			if !v {
+				cssFeature |= css
+			}
+		} else {
+			log.AddError(nil, logger.Range{}, fmt.Sprintf("%q is not a valid feature name for the \"supported\" setting", k))
+		}
+	}
+	return
+}
+
 func validateGlobalName(log logger.Log, text string) []string {
 	if text != "" {
 		source := logger.Source{
@@ -466,19 +475,21 @@ func validateLoaders(log logger.Log, loaders map[string]Loader) map[string]confi
 	return result
 }
 
-func validateJSXExpr(log logger.Log, text string, name string, kind js_parser.JSXExprKind) config.JSXExpr {
-	if expr, ok := js_parser.ParseJSXExpr(text, kind); ok {
-		return expr
+func validateJSXExpr(log logger.Log, text string, name string) config.DefineExpr {
+	if text != "" {
+		if expr, _ := js_parser.ParseDefineExprOrJSON(text); len(expr.Parts) > 0 || (name == "fragment" && expr.Constant != nil) {
+			return expr
+		}
+		log.AddError(nil, logger.Range{}, fmt.Sprintf("Invalid JSX %s: %q", name, text))
 	}
-	log.AddError(nil, logger.Range{}, fmt.Sprintf("Invalid JSX %s: %q", name, text))
-	return config.JSXExpr{}
+	return config.DefineExpr{}
 }
 
 func validateDefines(
 	log logger.Log,
 	defines map[string]string,
 	pureFns []string,
-	platform Platform,
+	platform config.Platform,
 	minify bool,
 	drop Drop,
 ) (*config.ProcessedDefines, []config.InjectedDefine) {
@@ -499,65 +510,31 @@ func validateDefines(
 			}
 		}
 
-		// Allow substituting for an identifier
-		if js_lexer.IsIdentifier(value) {
-			if _, ok := js_lexer.Keywords[value]; !ok {
-				switch value {
-				case "undefined":
-					rawDefines[key] = config.DefineData{
-						DefineFunc: func(config.DefineArgs) js_ast.E { return js_ast.EUndefinedShared },
-					}
-				case "NaN":
-					rawDefines[key] = config.DefineData{
-						DefineFunc: func(config.DefineArgs) js_ast.E { return &js_ast.ENumber{Value: math.NaN()} },
-					}
-				case "Infinity":
-					rawDefines[key] = config.DefineData{
-						DefineFunc: func(config.DefineArgs) js_ast.E { return &js_ast.ENumber{Value: math.Inf(1)} },
-					}
-				default:
-					name := value // The closure must close over a variable inside the loop
-					rawDefines[key] = config.DefineData{
-						DefineFunc: func(args config.DefineArgs) js_ast.E {
-							return &js_ast.EIdentifier{Ref: args.FindSymbol(args.Loc, name)}
-						},
-					}
-				}
-				continue
-			}
-		}
+		// Parse the value
+		defineExpr, injectExpr := js_parser.ParseDefineExprOrJSON(value)
 
-		// Parse the value as JSON
-		source := logger.Source{Contents: value}
-		expr, ok := js_parser.ParseJSON(logger.NewDeferLog(logger.DeferLogAll, nil), source, js_parser.JSONOptions{})
-		if !ok {
-			log.AddError(nil, logger.Range{}, fmt.Sprintf("Invalid define value (must be valid JSON syntax or a single identifier): %s", value))
+		// Define simple expressions
+		if defineExpr.Constant != nil || len(defineExpr.Parts) > 0 {
+			rawDefines[key] = config.DefineData{DefineExpr: &defineExpr}
 			continue
 		}
 
-		var fn config.DefineFunc
-		switch e := expr.Data.(type) {
-		// These values are inserted inline, and can participate in constant folding
-		case *js_ast.ENull:
-			fn = func(config.DefineArgs) js_ast.E { return js_ast.ENullShared }
-		case *js_ast.EBoolean:
-			fn = func(config.DefineArgs) js_ast.E { return &js_ast.EBoolean{Value: e.Value} }
-		case *js_ast.EString:
-			fn = func(config.DefineArgs) js_ast.E { return &js_ast.EString{Value: e.Value} }
-		case *js_ast.ENumber:
-			fn = func(config.DefineArgs) js_ast.E { return &js_ast.ENumber{Value: e.Value} }
-
-		// These values are extracted into a shared symbol reference
-		case *js_ast.EArray, *js_ast.EObject:
+		// Inject complex expressions
+		if injectExpr != nil {
 			definesToInject = append(definesToInject, key)
 			if valueToInject == nil {
 				valueToInject = make(map[string]config.InjectedDefine)
 			}
-			valueToInject[key] = config.InjectedDefine{Source: source, Data: e, Name: key}
+			valueToInject[key] = config.InjectedDefine{
+				Source: logger.Source{Contents: value},
+				Data:   injectExpr,
+				Name:   key,
+			}
 			continue
 		}
 
-		rawDefines[key] = config.DefineData{DefineFunc: fn}
+		// Anything else is unsupported
+		log.AddError(nil, logger.Range{}, fmt.Sprintf("Invalid define value (must be an entity name or valid JSON syntax): %s", value))
 	}
 
 	// Sort injected defines for determinism, since the imports will be injected
@@ -567,11 +544,8 @@ func validateDefines(
 		injectedDefines = make([]config.InjectedDefine, len(definesToInject))
 		sort.Strings(definesToInject)
 		for i, key := range definesToInject {
-			index := i // Capture this for the closure below
 			injectedDefines[i] = valueToInject[key]
-			rawDefines[key] = config.DefineData{DefineFunc: func(args config.DefineArgs) js_ast.E {
-				return &js_ast.EIdentifier{Ref: args.SymbolForDefine(index)}
-			}}
+			rawDefines[key] = config.DefineData{DefineExpr: &config.DefineExpr{InjectedDefineIndex: ast.MakeIndex32(uint32(i))}}
 		}
 	}
 
@@ -581,7 +555,7 @@ func validateDefines(
 	// that must be handled to avoid all React code crashing instantly. This
 	// is only done if it's not already defined so that you can override it if
 	// necessary.
-	if platform == PlatformBrowser {
+	if platform == config.PlatformBrowser {
 		if _, process := rawDefines["process"]; !process {
 			if _, processEnv := rawDefines["process.env"]; !processEnv {
 				if _, processEnvNodeEnv := rawDefines["process.env.NODE_ENV"]; !processEnvNodeEnv {
@@ -591,11 +565,7 @@ func validateDefines(
 					} else {
 						value = helpers.StringToUTF16("development")
 					}
-					rawDefines["process.env.NODE_ENV"] = config.DefineData{
-						DefineFunc: func(args config.DefineArgs) js_ast.E {
-							return &js_ast.EString{Value: value}
-						},
-					}
+					rawDefines["process.env.NODE_ENV"] = config.DefineData{DefineExpr: &config.DefineExpr{Constant: &js_ast.EString{Value: value}}}
 				}
 			}
 		}
@@ -706,6 +676,7 @@ func convertMessagesToPublic(kind logger.MsgKind, msgs []logger.Msg) []Message {
 				})
 			}
 			filtered = append(filtered, Message{
+				ID:         logger.MsgIDToString(msg.ID),
 				PluginName: msg.PluginName,
 				Text:       msg.Data.Text,
 				Location:   convertLocationToPublic(msg.Data.Location),
@@ -746,6 +717,7 @@ func convertMessagesToInternal(msgs []logger.Msg, kind logger.MsgKind, messages
 			})
 		}
 		msgs = append(msgs, logger.Msg{
+			ID:         logger.StringToMaximumMsgID(message.ID),
 			PluginName: message.PluginName,
 			Kind:       kind,
 			Data: logger.MsgData{
@@ -907,25 +879,34 @@ func rebuildImpl(
 		panic(err.Error())
 	}
 	targetFromAPI, jsFeatures, cssFeatures, targetEnv := validateFeatures(log, buildOpts.Target, buildOpts.Engines)
+	jsOverrides, jsMask, cssOverrides, cssMask := validateSupported(log, buildOpts.Supported)
 	outJS, outCSS := validateOutputExtensions(log, buildOpts.OutExtensions)
 	bannerJS, bannerCSS := validateBannerOrFooter(log, "banner", buildOpts.Banner)
 	footerJS, footerCSS := validateBannerOrFooter(log, "footer", buildOpts.Footer)
 	minify := buildOpts.MinifyWhitespace && buildOpts.MinifyIdentifiers && buildOpts.MinifySyntax
-	defines, injectedDefines := validateDefines(log, buildOpts.Define, buildOpts.Pure, buildOpts.Platform, minify, buildOpts.Drop)
+	platform := validatePlatform(buildOpts.Platform, config.PlatformBrowser)
+	defines, injectedDefines := validateDefines(log, buildOpts.Define, buildOpts.Pure, platform, minify, buildOpts.Drop)
 	mangleCache := cloneMangleCache(log, buildOpts.MangleCache)
 	options := config.Options{
-		TargetFromAPI:          targetFromAPI,
-		UnsupportedJSFeatures:  jsFeatures,
-		UnsupportedCSSFeatures: cssFeatures,
-		OriginalTargetEnv:      targetEnv,
+		TargetFromAPI:                      targetFromAPI,
+		UnsupportedJSFeatures:              jsFeatures.ApplyOverrides(jsOverrides, jsMask),
+		UnsupportedCSSFeatures:             cssFeatures.ApplyOverrides(cssOverrides, cssMask),
+		UnsupportedJSFeatureOverrides:      jsOverrides,
+		UnsupportedJSFeatureOverridesMask:  jsMask,
+		UnsupportedCSSFeatureOverrides:     cssOverrides,
+		UnsupportedCSSFeatureOverridesMask: cssMask,
+		OriginalTargetEnv:                  targetEnv,
 		JSX: config.JSXOptions{
-			Preserve: buildOpts.JSXMode == JSXModePreserve,
-			Factory:  validateJSXExpr(log, buildOpts.JSXFactory, "factory", js_parser.JSXFactory),
-			Fragment: validateJSXExpr(log, buildOpts.JSXFragment, "fragment", js_parser.JSXFragment),
+			Preserve:         buildOpts.JSXMode == JSXModePreserve,
+			AutomaticRuntime: buildOpts.JSXMode == JSXModeAutomatic,
+			Factory:          validateJSXExpr(log, buildOpts.JSXFactory, "factory"),
+			Fragment:         validateJSXExpr(log, buildOpts.JSXFragment, "fragment"),
+			Development:      buildOpts.JSXDev,
+			ImportSource:     buildOpts.JSXImportSource,
 		},
 		Defines:               defines,
 		InjectedDefines:       injectedDefines,
-		Platform:              validatePlatform(buildOpts.Platform),
+		Platform:              platform,
 		SourceMap:             validateSourceMap(buildOpts.Sourcemap),
 		LegalComments:         validateLegalComments(buildOpts.LegalComments, buildOpts.Bundle),
 		SourceRoot:            buildOpts.SourceRoot,
@@ -1024,6 +1005,10 @@ func rebuildImpl(
 				log.AddError(nil, logger.Range{}, "Cannot use the \"file\" loader without an output path")
 				break
 			}
+			if loader == config.LoaderCopy {
+				log.AddError(nil, logger.Range{}, "Cannot use the \"copy\" loader without an output path")
+				break
+			}
 		}
 
 		// Use the current directory as the output directory instead of an empty
@@ -1374,13 +1359,17 @@ func transformImpl(input string, transformOpts TransformOptions) TransformResult
 	var unusedImportFlagsTS config.UnusedImportFlagsTS
 	useDefineForClassFieldsTS := config.Unspecified
 	jsx := config.JSXOptions{
-		Preserve: transformOpts.JSXMode == JSXModePreserve,
-		Factory:  validateJSXExpr(log, transformOpts.JSXFactory, "factory", js_parser.JSXFactory),
-		Fragment: validateJSXExpr(log, transformOpts.JSXFragment, "fragment", js_parser.JSXFragment),
+		Preserve:         transformOpts.JSXMode == JSXModePreserve,
+		AutomaticRuntime: transformOpts.JSXMode == JSXModeAutomatic,
+		Factory:          validateJSXExpr(log, transformOpts.JSXFactory, "factory"),
+		Fragment:         validateJSXExpr(log, transformOpts.JSXFragment, "fragment"),
+		Development:      transformOpts.JSXDev,
+		ImportSource:     transformOpts.JSXImportSource,
 	}
 
 	// Settings from "tsconfig.json" override those
 	var tsTarget *config.TSTarget
+	var tsAlwaysStrict *config.TSAlwaysStrict
 	caches := cache.MakeCacheSet()
 	if transformOpts.TsconfigRaw != "" {
 		source := logger.Source{
@@ -1389,11 +1378,17 @@ func transformImpl(input string, transformOpts TransformOptions) TransformResult
 			Contents:   transformOpts.TsconfigRaw,
 		}
 		if result := resolver.ParseTSConfigJSON(log, source, &caches.JSONCache, nil); result != nil {
+			if result.JSX != config.TSJSXNone {
+				jsx.SetOptionsFromTSJSX(result.JSX)
+			}
 			if len(result.JSXFactory) > 0 {
-				jsx.Factory = config.JSXExpr{Parts: result.JSXFactory}
+				jsx.Factory = config.DefineExpr{Parts: result.JSXFactory}
 			}
 			if len(result.JSXFragmentFactory) > 0 {
-				jsx.Fragment = config.JSXExpr{Parts: result.JSXFragmentFactory}
+				jsx.Fragment = config.DefineExpr{Parts: result.JSXFragmentFactory}
+			}
+			if len(result.JSXImportSource) > 0 {
+				jsx.ImportSource = result.JSXImportSource
 			}
 			if result.UseDefineForClassFields != config.Unspecified {
 				useDefineForClassFieldsTS = result.UseDefineForClassFields
@@ -1403,6 +1398,7 @@ func transformImpl(input string, transformOpts TransformOptions) TransformResult
 				result.PreserveValueImports,
 			)
 			tsTarget = result.TSTarget
+			tsAlwaysStrict = result.TSAlwaysStrict
 		}
 	}
 
@@ -1416,37 +1412,45 @@ func transformImpl(input string, transformOpts TransformOptions) TransformResult
 
 	// Convert and validate the transformOpts
 	targetFromAPI, jsFeatures, cssFeatures, targetEnv := validateFeatures(log, transformOpts.Target, transformOpts.Engines)
-	defines, injectedDefines := validateDefines(log, transformOpts.Define, transformOpts.Pure, PlatformNeutral, false /* minify */, transformOpts.Drop)
+	jsOverrides, jsMask, cssOverrides, cssMask := validateSupported(log, transformOpts.Supported)
+	platform := validatePlatform(transformOpts.Platform, config.PlatformNeutral)
+	defines, injectedDefines := validateDefines(log, transformOpts.Define, transformOpts.Pure, platform, false /* minify */, transformOpts.Drop)
 	mangleCache := cloneMangleCache(log, transformOpts.MangleCache)
 	options := config.Options{
-		TargetFromAPI:           targetFromAPI,
-		UnsupportedJSFeatures:   jsFeatures,
-		UnsupportedCSSFeatures:  cssFeatures,
-		OriginalTargetEnv:       targetEnv,
-		TSTarget:                tsTarget,
-		JSX:                     jsx,
-		Defines:                 defines,
-		InjectedDefines:         injectedDefines,
-		SourceMap:               validateSourceMap(transformOpts.Sourcemap),
-		LegalComments:           validateLegalComments(transformOpts.LegalComments, false /* bundle */),
-		SourceRoot:              transformOpts.SourceRoot,
-		ExcludeSourcesContent:   transformOpts.SourcesContent == SourcesContentExclude,
-		OutputFormat:            validateFormat(transformOpts.Format),
-		GlobalName:              validateGlobalName(log, transformOpts.GlobalName),
-		MinifySyntax:            transformOpts.MinifySyntax,
-		MinifyWhitespace:        transformOpts.MinifyWhitespace,
-		MinifyIdentifiers:       transformOpts.MinifyIdentifiers,
-		MangleProps:             validateRegex(log, "mangle props", transformOpts.MangleProps),
-		ReserveProps:            validateRegex(log, "reserve props", transformOpts.ReserveProps),
-		MangleQuoted:            transformOpts.MangleQuoted == MangleQuotedTrue,
-		DropDebugger:            (transformOpts.Drop & DropDebugger) != 0,
-		ASCIIOnly:               validateASCIIOnly(transformOpts.Charset),
-		IgnoreDCEAnnotations:    transformOpts.IgnoreAnnotations,
-		TreeShaking:             validateTreeShaking(transformOpts.TreeShaking, false /* bundle */, transformOpts.Format),
-		AbsOutputFile:           transformOpts.Sourcefile + "-out",
-		KeepNames:               transformOpts.KeepNames,
-		UseDefineForClassFields: useDefineForClassFieldsTS,
-		UnusedImportFlagsTS:     unusedImportFlagsTS,
+		TargetFromAPI:                      targetFromAPI,
+		UnsupportedJSFeatures:              jsFeatures.ApplyOverrides(jsOverrides, jsMask),
+		UnsupportedCSSFeatures:             cssFeatures.ApplyOverrides(cssOverrides, cssMask),
+		UnsupportedJSFeatureOverrides:      jsOverrides,
+		UnsupportedJSFeatureOverridesMask:  jsMask,
+		UnsupportedCSSFeatureOverrides:     cssOverrides,
+		UnsupportedCSSFeatureOverridesMask: cssMask,
+		OriginalTargetEnv:                  targetEnv,
+		TSTarget:                           tsTarget,
+		TSAlwaysStrict:                     tsAlwaysStrict,
+		JSX:                                jsx,
+		Defines:                            defines,
+		InjectedDefines:                    injectedDefines,
+		Platform:                           platform,
+		SourceMap:                          validateSourceMap(transformOpts.Sourcemap),
+		LegalComments:                      validateLegalComments(transformOpts.LegalComments, false /* bundle */),
+		SourceRoot:                         transformOpts.SourceRoot,
+		ExcludeSourcesContent:              transformOpts.SourcesContent == SourcesContentExclude,
+		OutputFormat:                       validateFormat(transformOpts.Format),
+		GlobalName:                         validateGlobalName(log, transformOpts.GlobalName),
+		MinifySyntax:                       transformOpts.MinifySyntax,
+		MinifyWhitespace:                   transformOpts.MinifyWhitespace,
+		MinifyIdentifiers:                  transformOpts.MinifyIdentifiers,
+		MangleProps:                        validateRegex(log, "mangle props", transformOpts.MangleProps),
+		ReserveProps:                       validateRegex(log, "reserve props", transformOpts.ReserveProps),
+		MangleQuoted:                       transformOpts.MangleQuoted == MangleQuotedTrue,
+		DropDebugger:                       (transformOpts.Drop & DropDebugger) != 0,
+		ASCIIOnly:                          validateASCIIOnly(transformOpts.Charset),
+		IgnoreDCEAnnotations:               transformOpts.IgnoreAnnotations,
+		TreeShaking:                        validateTreeShaking(transformOpts.TreeShaking, false /* bundle */, transformOpts.Format),
+		AbsOutputFile:                      transformOpts.Sourcefile + "-out",
+		KeepNames:                          transformOpts.KeepNames,
+		UseDefineForClassFields:            useDefineForClassFieldsTS,
+		UnusedImportFlagsTS:                unusedImportFlagsTS,
 		Stdin: &config.StdinInfo{
 			Loader:     validateLoader(transformOpts.Loader),
 			Contents:   input,
@@ -1487,7 +1491,7 @@ func transformImpl(input string, transformOpts TransformOptions) TransformResult
 		}
 
 		// Scan over the bundle
-		mockFS := fs.MockFS(make(map[string]string))
+		mockFS := fs.MockFS(make(map[string]string), fs.MockUnix)
 		resolver := resolver.NewResolver(mockFS, log, caches, options)
 		bundle := bundler.ScanBundle(log, mockFS, resolver, caches, nil, options, timer)
 
