@@ -23,6 +23,7 @@ func newBuildOptions() api.BuildOptions {
 		Footer:      make(map[string]string),
 		Loader:      make(map[string]api.Loader),
 		LogOverride: make(map[string]api.LogLevel),
+		Supported:   make(map[string]bool),
 	}
 }
 
@@ -30,6 +31,7 @@ func newTransformOptions() api.TransformOptions {
 	return api.TransformOptions{
 		Define:      make(map[string]string),
 		LogOverride: make(map[string]api.LogLevel),
+		Supported:   make(map[string]bool),
 	}
 }
 
@@ -457,6 +459,24 @@ func parseOptionsImpl(
 				transformOpts.LogOverride[value[:equals]] = logLevel
 			}
 
+		case strings.HasPrefix(arg, "--supported:"):
+			value := arg[len("--supported:"):]
+			equals := strings.IndexByte(value, '=')
+			if equals == -1 {
+				return parseOptionsExtras{}, cli_helpers.MakeErrorWithNote(
+					fmt.Sprintf("Missing \"=\" in %q", arg),
+					"You need to use \"=\" to specify both the name of the feature and whether it is supported or not. "+
+						"For example, \"--supported:arrow=false\" marks arrow functions as unsupported.",
+				)
+			}
+			if isSupported, err := parseBoolFlag(arg, true); err != nil {
+				return parseOptionsExtras{}, err
+			} else if buildOpts != nil {
+				buildOpts.Supported[value[:equals]] = isSupported
+			} else {
+				transformOpts.Supported[value[:equals]] = isSupported
+			}
+
 		case strings.HasPrefix(arg, "--pure:"):
 			value := arg[len("--pure:"):]
 			if buildOpts != nil {
@@ -488,11 +508,11 @@ func parseOptionsImpl(
 			if err != nil {
 				return parseOptionsExtras{}, err
 			}
-			if loader == api.LoaderFile {
+			if loader == api.LoaderFile || loader == api.LoaderCopy {
 				return parseOptionsExtras{}, cli_helpers.MakeErrorWithNote(
 					fmt.Sprintf("%q is not supported when transforming stdin", arg),
-					"Using esbuild to transform stdin only generates one output file, so you cannot use the \"file\" loader "+
-						"since that needs to generate two output files.",
+					fmt.Sprintf("Using esbuild to transform stdin only generates one output file, so you cannot use the %q loader "+
+						"since that needs to generate two output files.", value),
 				)
 			}
 			if buildOpts != nil {
@@ -532,49 +552,49 @@ func parseOptionsImpl(
 			}
 			buildOpts.OutExtensions[value[:equals]] = value[equals+1:]
 
-		case strings.HasPrefix(arg, "--platform=") && buildOpts != nil:
+		case strings.HasPrefix(arg, "--platform="):
 			value := arg[len("--platform="):]
+			var platform api.Platform
 			switch value {
 			case "browser":
-				buildOpts.Platform = api.PlatformBrowser
+				platform = api.PlatformBrowser
 			case "node":
-				buildOpts.Platform = api.PlatformNode
+				platform = api.PlatformNode
 			case "neutral":
-				buildOpts.Platform = api.PlatformNeutral
+				platform = api.PlatformNeutral
 			default:
 				return parseOptionsExtras{}, cli_helpers.MakeErrorWithNote(
 					fmt.Sprintf("Invalid value %q in %q", value, arg),
 					"Valid values are \"browser\", \"node\", or \"neutral\".",
 				)
 			}
+			if buildOpts != nil {
+				buildOpts.Platform = platform
+			} else {
+				transformOpts.Platform = platform
+			}
 
 		case strings.HasPrefix(arg, "--format="):
 			value := arg[len("--format="):]
+			var format api.Format
 			switch value {
 			case "iife":
-				if buildOpts != nil {
-					buildOpts.Format = api.FormatIIFE
-				} else {
-					transformOpts.Format = api.FormatIIFE
-				}
+				format = api.FormatIIFE
 			case "cjs":
-				if buildOpts != nil {
-					buildOpts.Format = api.FormatCommonJS
-				} else {
-					transformOpts.Format = api.FormatCommonJS
-				}
+				format = api.FormatCommonJS
 			case "esm":
-				if buildOpts != nil {
-					buildOpts.Format = api.FormatESModule
-				} else {
-					transformOpts.Format = api.FormatESModule
-				}
+				format = api.FormatESModule
 			default:
 				return parseOptionsExtras{}, cli_helpers.MakeErrorWithNote(
 					fmt.Sprintf("Invalid value %q in %q", value, arg),
 					"Valid values are \"iife\", \"cjs\", or \"esm\".",
 				)
 			}
+			if buildOpts != nil {
+				buildOpts.Format = format
+			} else {
+				transformOpts.Format = format
+			}
 
 		case strings.HasPrefix(arg, "--external:") && buildOpts != nil:
 			buildOpts.External = append(buildOpts.External, arg[len("--external:"):])
@@ -590,10 +610,12 @@ func parseOptionsImpl(
 				mode = api.JSXModeTransform
 			case "preserve":
 				mode = api.JSXModePreserve
+			case "automatic":
+				mode = api.JSXModeAutomatic
 			default:
 				return parseOptionsExtras{}, cli_helpers.MakeErrorWithNote(
 					fmt.Sprintf("Invalid value %q in %q", value, arg),
-					"Valid values are \"transform\" or \"preserve\".",
+					"Valid values are \"transform\", \"automatic\", or \"preserve\".",
 				)
 			}
 			if buildOpts != nil {
@@ -618,6 +640,23 @@ func parseOptionsImpl(
 				transformOpts.JSXFragment = value
 			}
 
+		case strings.HasPrefix(arg, "--jsx-import-source="):
+			value := arg[len("--jsx-import-source="):]
+			if buildOpts != nil {
+				buildOpts.JSXImportSource = value
+			} else {
+				transformOpts.JSXImportSource = value
+			}
+
+		case isBoolFlag(arg, "--jsx-dev"):
+			if value, err := parseBoolFlag(arg, true); err != nil {
+				return parseOptionsExtras{}, err
+			} else if buildOpts != nil {
+				buildOpts.JSXDev = value
+			} else {
+				transformOpts.JSXDev = value
+			}
+
 		case strings.HasPrefix(arg, "--banner=") && transformOpts != nil:
 			transformOpts.Banner = arg[len("--banner="):]
 
@@ -714,6 +753,7 @@ func parseOptionsImpl(
 				"allow-overwrite":    true,
 				"bundle":             true,
 				"ignore-annotations": true,
+				"jsx-dev":            true,
 				"keep-names":         true,
 				"minify-identifiers": true,
 				"minify-syntax":      true,
@@ -741,6 +781,7 @@ func parseOptionsImpl(
 				"ignore-annotations": true,
 				"jsx-factory":        true,
 				"jsx-fragment":       true,
+				"jsx-import-source":  true,
 				"jsx":                true,
 				"keep-names":         true,
 				"legal-comments":     true,
@@ -787,6 +828,7 @@ func parseOptionsImpl(
 				"log-override":  true,
 				"out-extension": true,
 				"pure":          true,
+				"supported":     true,
 			}
 
 			note := ""
@@ -865,17 +907,6 @@ func parseTargets(targets []string, arg string) (target api.Target, engines []ap
 		"es2022": api.ES2022,
 	}
 
-	validEngines := map[string]api.EngineName{
-		"chrome":  api.EngineChrome,
-		"edge":    api.EngineEdge,
-		"firefox": api.EngineFirefox,
-		"ie":      api.EngineIE,
-		"ios":     api.EngineIOS,
-		"node":    api.EngineNode,
-		"opera":   api.EngineOpera,
-		"safari":  api.EngineSafari,
-	}
-
 outer:
 	for _, value := range targets {
 		if valid, ok := validTargets[strings.ToLower(value)]; ok {
