@@ -81,11 +81,31 @@ const buildNeutralLib = (esbuildPath) => {
   fs.writeFileSync(pjPath, JSON.stringify(package_json, null, 2) + '\n')
 }
 
-exports.buildWasmLib = async (esbuildPath) => {
-  const npmWasmShimDirs = [
-    path.join(repoDir, 'npm', 'esbuild-android-64'),
-  ]
+function generateWorkerCode({ esbuildPath, wasm_exec_js, minify, target }) {
+  const input = `
+    let onmessage;
+    let globalThis = {};
+    for (let o = self; o; o = Object.getPrototypeOf(o))
+      for (let k of Object.getOwnPropertyNames(o))
+        if (!(k in globalThis))
+          Object.defineProperty(globalThis, k, { get: () => self[k] });
+    ${wasm_exec_js.replace(/\bfs\./g, 'globalThis.fs.')}
+    ${fs.readFileSync(path.join(repoDir, 'lib', 'shared', 'worker.ts'), 'utf8')}
+    return m => onmessage(m)
+  `;
+  const wasmExecAndWorker = childProcess.execFileSync(esbuildPath, [
+    '--loader=ts',
+    '--target=' + target,
+    '--define:ESBUILD_VERSION=' + JSON.stringify(version),
+  ].concat(minify ? ['--minify'] : []), { cwd: repoDir, input }).toString().trim()
+  const commentLines = wasm_exec_js.split('\n')
+  const firstNonComment = commentLines.findIndex(line => !line.startsWith('//'))
+  const commentPrefix = '\n' + commentLines.slice(0, firstNonComment).join('\n') + '\n'
+  if (minify) return `(postMessage=>{${commentPrefix}${wasmExecAndWorker}})`
+  return `((postMessage) => {${(commentPrefix + wasmExecAndWorker).replace(/\n/g, '\n      ')}\n    })`
+}
 
+exports.buildWasmLib = async (esbuildPath) => {
   // Asynchronously start building the WebAssembly module
   const npmWasmDir = path.join(repoDir, 'npm', 'esbuild-wasm')
   const goBuildPromise = new Promise((resolve, reject) => childProcess.execFile('go',
@@ -133,33 +153,8 @@ exports.buildWasmLib = async (esbuildPath) => {
 
   for (const minify of [false, true]) {
     const minifyFlags = minify ? ['--minify'] : []
-    let wasmWorkerCode = {}
-
-    for (const [format, target] of Object.entries({ umd: umdBrowserTarget, esm: esmBrowserTarget })) {
-      // Process "npm/esbuild-wasm/wasm_exec.js" and "lib/worker.ts"
-      const input = `
-        let onmessage;
-        let globalThis = {};
-        for (let o = self; o; o = Object.getPrototypeOf(o))
-          for (let k of Object.getOwnPropertyNames(o))
-            if (!(k in globalThis))
-              Object.defineProperty(globalThis, k, { get: () => self[k] });
-        ${wasm_exec_js.replace(/\bfs\./g, 'globalThis.fs.')}
-        ${fs.readFileSync(path.join(repoDir, 'lib', 'npm', 'worker.ts'), 'utf8')}
-        return m => onmessage(m)
-      `;
-      const wasmExecAndWorker = childProcess.execFileSync(esbuildPath, [
-        '--loader=ts',
-        '--target=' + target,
-        '--define:ESBUILD_VERSION=' + JSON.stringify(version),
-      ].concat(minifyFlags), { cwd: repoDir, input }).toString().trim()
-      const commentLines = wasm_exec_js.split('\n')
-      const firstNonComment = commentLines.findIndex(line => !line.startsWith('//'))
-      const commentPrefix = '\n' + commentLines.slice(0, firstNonComment).join('\n') + '\n'
-      wasmWorkerCode[format] = minify
-        ? `(postMessage=>{${commentPrefix}${wasmExecAndWorker}})`
-        : `((postMessage) => {${(commentPrefix + wasmExecAndWorker).replace(/\n/g, '\n      ')}\n    })`
-    }
+    const wasmWorkerCodeUMD = generateWorkerCode({ esbuildPath, wasm_exec_js, minify, target: umdBrowserTarget })
+    const wasmWorkerCodeESM = generateWorkerCode({ esbuildPath, wasm_exec_js, minify, target: esmBrowserTarget })
 
     // Generate "npm/esbuild-wasm/lib/browser.*"
     const umdPrefix = `(module=>{`
@@ -170,11 +165,11 @@ exports.buildWasmLib = async (esbuildPath) => {
       '--target=' + umdBrowserTarget,
       '--format=cjs',
       '--define:ESBUILD_VERSION=' + JSON.stringify(version),
-      '--define:WEB_WORKER_SOURCE_CODE=' + JSON.stringify(wasmWorkerCode.umd),
+      '--define:WEB_WORKER_SOURCE_CODE=' + JSON.stringify(wasmWorkerCodeUMD),
       '--banner:js=' + umdPrefix,
       '--footer:js=' + umdSuffix,
       '--log-level=warning',
-    ].concat(minifyFlags), { cwd: repoDir }).toString().replace('WEB_WORKER_FUNCTION', wasmWorkerCode.umd)
+    ].concat(minifyFlags), { cwd: repoDir }).toString().replace('WEB_WORKER_FUNCTION', wasmWorkerCodeUMD)
     fs.writeFileSync(path.join(libDir, minify ? 'browser.min.js' : 'browser.js'), browserCJS)
 
     // Generate "npm/esbuild-wasm/esm/browser.min.js"
@@ -184,9 +179,9 @@ exports.buildWasmLib = async (esbuildPath) => {
       '--target=' + esmBrowserTarget,
       '--format=esm',
       '--define:ESBUILD_VERSION=' + JSON.stringify(version),
-      '--define:WEB_WORKER_SOURCE_CODE=' + JSON.stringify(wasmWorkerCode.esm),
+      '--define:WEB_WORKER_SOURCE_CODE=' + JSON.stringify(wasmWorkerCodeESM),
       '--log-level=warning',
-    ].concat(minifyFlags), { cwd: repoDir }).toString().replace('WEB_WORKER_FUNCTION', wasmWorkerCode.esm)
+    ].concat(minifyFlags), { cwd: repoDir }).toString().replace('WEB_WORKER_FUNCTION', wasmWorkerCodeESM)
     fs.writeFileSync(path.join(esmDir, minify ? 'browser.min.js' : 'browser.js'), browserESM)
   }
 
@@ -214,7 +209,9 @@ module.exports = ${JSON.stringify(exit0Map, null, 2)};
   await goBuildPromise;
 
   // Also copy this into the WebAssembly shim directories
-  for (const dir of npmWasmShimDirs) {
+  for (const dir of [
+    path.join(repoDir, 'npm', 'esbuild-android-64'),
+  ]) {
     fs.mkdirSync(path.join(dir, 'bin'), { recursive: true })
     fs.writeFileSync(path.join(dir, 'wasm_exec.js'), wasm_exec_js);
     fs.writeFileSync(path.join(dir, 'wasm_exec_node.js'), wasm_exec_node_js);
@@ -237,14 +234,34 @@ const buildDenoLib = (esbuildPath) => {
     '--banner:js=/// <reference types="./mod.d.ts" />',
   ], { cwd: repoDir })
 
+  // Generate "deno/esbuild/wasm.js"
+  const GOROOT = childProcess.execFileSync('go', ['env', 'GOROOT']).toString().trim();
+  let wasm_exec_js = fs.readFileSync(path.join(GOROOT, 'misc', 'wasm', 'wasm_exec.js'), 'utf8');
+  const wasmWorkerCode = generateWorkerCode({ esbuildPath, wasm_exec_js, minify: true, target: 'esnext' })
+  const modWASM = childProcess.execFileSync(esbuildPath, [
+    path.join(repoDir, 'lib', 'deno', 'wasm.ts'),
+    '--bundle',
+    '--target=esnext',
+    '--define:ESBUILD_VERSION=' + JSON.stringify(version),
+    '--define:WEB_WORKER_SOURCE_CODE=' + JSON.stringify(wasmWorkerCode),
+    '--platform=neutral',
+    '--log-level=warning',
+    '--banner:js=/// <reference types="./wasm.d.ts" />',
+  ], { cwd: repoDir }).toString().replace('WEB_WORKER_FUNCTION', wasmWorkerCode)
+  fs.writeFileSync(path.join(denoDir, 'wasm.js'), modWASM)
+
   // Generate "deno/esbuild/mod.d.ts"
-  const types_ts = fs.readFileSync(path.join(repoDir, 'lib', 'shared', 'types.ts'), 'utf8')
-  fs.writeFileSync(path.join(denoDir, 'mod.d.ts'), types_ts +
+  const types_ts = fs.readFileSync(path.join(repoDir, 'lib', 'shared', 'types.ts'), 'utf8') +
     `\n// Unlike node, Deno lacks the necessary APIs to clean up child processes` +
     `\n// automatically. You must manually call stop() in Deno when you're done` +
     `\n// using esbuild or Deno will continue running forever.` +
     `\nexport function stop(): void;` +
-    `\n`)
+    `\n`
+  fs.writeFileSync(path.join(denoDir, 'mod.d.ts'), types_ts)
+  fs.writeFileSync(path.join(denoDir, 'wasm.d.ts'), types_ts)
+
+  // And copy the WebAssembly file over to the Deno library as well
+  fs.copyFileSync(path.join(repoDir, 'npm', 'esbuild-wasm', 'esbuild.wasm'), path.join(repoDir, 'deno', 'esbuild.wasm'));
 }
 
 // Writing a file atomically is important for watch mode tests since we don't
